import Pyro.core
import Pyro.naming
import Pyro.EventService.Server
import socket
from Pyro.errors import *

# SUBSCRIBER: subscribes to certain events.
class Subscriber(Pyro.core.ObjBase):
	def __init__(self):
		Pyro.core.ObjBase.__init__(self)
		Pyro.core.initServer()
		Pyro.core.initClient()
		self.daemon = Pyro.core.Daemon()
		locator = Pyro.naming.NameServerLocator()
		# find the NS
		try:
			self.NS = locator.getNS()
		except (PyroError, socket.error),x:
			hn = socket.gethostname()
			self.NS = locator.getNS(host=hn)

		self.daemon.useNameServer(self.NS)
		self.daemon.connect(self)

	def subscribe(self,subjects):
		# Subscribe to one or more subjects.
		# It is safe to call this multiple times.
		uri = self.NS.resolve(Pyro.EventService.Server.EVENTSERVICE_NAME)
		eventservice=Pyro.core.getProxyForURI(uri)
		eventservice.subscribe(subjects, self.getProxy())
	def subscribeMatch(self,subjectPatterns):
		# Subscribe to one or more subjects (by pattern)
		# It is safe to call this multiple times.
		uri = self.NS.resolve(Pyro.EventService.Server.EVENTSERVICE_NAME)
		eventservice=Pyro.core.getProxyForURI(uri)
		eventservice.subscribeMatch(subjectPatterns, self.getProxy())

	def listen(self):
		while 1:
			self.daemon.handleRequests()

	def event(self, event):					# callback, override this!
		print event

# PUBLISHER: publishes events.
class Publisher:
	def __init__(self):
		Pyro.core.initClient()
		locator = Pyro.naming.NameServerLocator()
		try:
			ns = locator.getNS()
		except (PyroError, socket.error),x:
			hn = socket.gethostname()
			ns = locator.getNS(host=hn)

		uri = ns.resolve(Pyro.EventService.Server.EVENTSERVICE_NAME)
		self.eventservice=Pyro.core.getProxyForURI(uri)

	def publish(self, subjects, msg):
		self.eventservice.publish(subjects,msg)

