import Pyro.core
import Pyro.naming
from Pyro.errors import *
import time, types, re, socket

EVENTSERVICE_NAME = ":Pyro.EventService"
EVENTSERVICE_VERSION = "1.0"

# EVENT - the thing that is published. Has a subject and contains a message.
class Event:
	def __init__(self, subject, msg):
		self.msg=msg
		self.subject=subject
		self.time=time.time()
	def __str__(self):
		return "<EVENT SUBJ %s (%s): %s>" % (self.subject, time.ctime(self.time), str(self.msg))


# The EVENTSERVICE is the actual Pyro server.
class EventService(Pyro.core.ObjBase):
	def __init__(self):
		Pyro.core.ObjBase.__init__(self)
		self.subscribers={}			# subject -> { subscriber -> dummy }
		self.subscribersMatch={}	# subjectPattern -> { subscriber -> dummy }
	def _mksequence(self, seq):
		if not (type(seq) in (types.TupleType,types.ListType)):
			seq=(seq,)
		return seq
	def subscribe(self, subjects, subscriber):
		# Subscribe into a dictionary; this way; somebody can subscribe
		# only once to this subject. Subjects are exact strings.
		for subject in self._mksequence(subjects):
			self.subscribers.setdefault(subject.lower(),{}) [subscriber]=1
	def subscribeMatch(self, subjects, subscriber):
		# Subscribe into a dictionary; this way; somebody can subscribe
		# only once to this subject. Subjects are regex patterns.
		for subject in self._mksequence(subjects):
			matcher = re.compile(subject,re.IGNORECASE)
			self.subscribersMatch.setdefault(matcher,{}) [subscriber]=1
	def unsubscribe(self, subjects, subscriber):
		for subject in self._mksequence(subjects):
			try:
				del self.subscribers[subject.lower()] [subscriber]
			except KeyError:
				try:
					m=re.compile(subject,re.IGNORECASE)
					del self.subscribersMatch[m] [subscriber]
				except KeyError:
					pass
	def publish(self, subjects, message):
		# publish a message. Subjects must be exact strings
		for subject in self._mksequence(subjects):
			event = Event(subject, message)
			try:
				for s in self.subscribers[subject.lower()].keys():
					try:
						s.event(event)
					except SocketClosedError:
						# hmm, subscriber no longer connected...
						del self.subscribers[subject.lower()] [s]
					except:
						# ignore all other errors
						pass
			except KeyError:
				pass
			# process the subject patterns
			for (m,subs) in self.subscribersMatch.items():
				if m.match(subject):
					# send event to all subscribers
					for s in subs.keys():	
						try:
							s.event(event)
						except SocketClosedError:
							# hmm, subscriber no longer connected...
							del self.subscribersMatch[m] [s]
						except:
							# ignore all other errors
							pass

def start():
	print '*** Pyro Event Service ***'
	Pyro.core.initServer()
	PyroDaemon = Pyro.core.Daemon()
	locator = Pyro.naming.NameServerLocator()
	try:
		print 'Looking for Name Server...'
		ns = locator.getNS()
	except (PyroError, socket.error),x:
		hn = socket.gethostname()
		ns = locator.getNS(host=hn)

	PyroDaemon.useNameServer(ns)
	try:
		ns.unregister(EVENTSERVICE_NAME)
	except NamingError:
		pass

	PyroDaemon.connect(EventService(), EVENTSERVICE_NAME)
	print 'Event service V'+EVENTSERVICE_VERSION,'is running.'
	while 1:
		PyroDaemon.handleRequests()

