#! /usr/bin/env python
import sys, socket
import Pyro.naming, Pyro.core, Pyro.util, Pyro.protocol
from Pyro.errors import PyroError,NamingError


######## Custom connections validator.
######## This validator asks the user to accept or deny a new connection.
######## Note that the old validator is still called when the user accepts.

class hostCheckingValidator(Pyro.protocol.limitingConnValidator):
	def __init__(self, daemon):
		Pyro.protocol.limitingConnValidator.__init__(self, daemon)
	def accept(self,conn):
		(ip, port)=conn.addr
		try:
			hostname=socket.gethostbyaddr(ip)[0]
		except:
			hostname='<unknown>'
		print '\nNew connection from',ip,hostname
		a=raw_input('Do you want to accept this? y/n: ')
		if a=='y':
			# user accepts, but pass it on to the default validator,
			# which will check the max. number of connections...
			return Pyro.protocol.limitingConnValidator.accept(self, conn)
		else:
			Pyro.util.Log.msg('ConnValidator','User denied connection from',ip,hostname)
			return (0,2)	# not ok, code 2 (host denied)


##### test object

class testobject(Pyro.core.ObjBase):
	def method(self,arg):
		print 'method was called with',arg
		return 42


##### main program.

# initialize the server and set the default namespace group
Pyro.core.initServer()
Pyro.config.PYRO_TRACELEVEL=3
Pyro.config.PYRO_NS_DEFAULTGROUP=':test'
Pyro.config.PYRO_LOGFILE='server_log'
print 'Check the logfile for messages: server_log'

# Construct the Pyro Daemon with our own connection validator
PyroDaemon = Pyro.core.Daemon()
PyroDaemon.setNewConnectionValidator(hostCheckingValidator(PyroDaemon))		### <<--- mind this!!!

# locate the NS
locator = Pyro.naming.NameServerLocator()
print 'searching for Naming Service...'
try:
	ns = locator.getNS()
except (PyroError,socket.error),x:
	hn = socket.gethostname()
	print '\nNaming Service not found with broadcast. Trying host',hn,'...',
	ns = locator.getNS(host=hn)

print 'Naming Service found at',ns.URI.address,'('+(Pyro.protocol.getHostname(ns.URI.address) or '??')+') port',ns.URI.port

# make sure our namespace group exists
try: ns.createGroup(':test')
except NamingError: pass

PyroDaemon.useNameServer(ns)

# connect a new object implementation (first unregister previous one)
try: ns.unregister('denyhosts')
except NamingError: pass

PyroDaemon.connect(testobject(),'denyhosts')

# enter the service loop.
print 'Server object "denyhosts" ready.'
while 1:
	PyroDaemon.handleRequests(3.0)
	sys.stdout.write('.')
	sys.stdout.flush()

